@@ -1,7 +1,7 @@
 package Test::Script;
 
 # ABSTRACT: Basic cross-platform tests for scripts
-our $VERSION = '1.09'; # VERSION
+our $VERSION = '1.12'; # VERSION
 
 
 use 5.006;
@@ -12,8 +12,10 @@ use Exporter         ();
 use File::Spec       ();
 use File::Spec::Unix ();
 use Probe::Perl      ();
-use IPC::Run3        ();
+use IPC::Run3        qw( run3 );
 use Test::Builder    ();
+use File::Temp       ();
+use File::Path       ();
 
 our @ISA     = 'Exporter';
 our @EXPORT  = qw{
@@ -62,6 +64,24 @@ sub path ($) {
   );
 }
 
+## This can and should be removed if/when IPC::Run3 is fixed on MSWin32
+## See rt94685, rt46333, rt95308 and IPC-Run3/gh#9"
+sub _borked_ipc_run3 () {
+  $^O eq 'MSWin32' &&
+  ! eval { IPC::Run3::run3 [ perl, -e => 'BEGIN {die}' ], \undef, \undef, \undef; 1 }
+}
+
+if(_borked_ipc_run3())
+{
+  no warnings 'redefine';
+  *run3 = sub {
+    $! = 0;
+    my $r = IPC::Run3::run3(@_, { return_if_system_error => 1 });
+    Carp::croak($!) if $! && $! !~ /Inappropriate I\/O control operation/;
+    $r;
+  };
+}
+
 #####################################################################
 # Test Functions
 
@@ -71,11 +91,12 @@ sub script_compiles {
   my $unix   = shift @$args;
   my $path   = path( $unix );
   my @libs   = map { "-I$_" } grep {!ref($_)} @INC;
-  my $cmd    = [ perl, @libs, '-c', $path, @$args ];
+  my $dir    = _preload_module();
+  my $cmd    = [ perl, "-I$dir", '-M__TEST_SCRIPT__', '-c', $path, @$args ];
   my $stdin  = '';
   my $stdout = '';
   my $stderr = '';
-  my $rv     = eval { IPC::Run3::run3( $cmd, \$stdin, \$stdout, \$stderr ) };
+  my $rv     = eval { run3( $cmd, \$stdin, \$stdout, \$stderr ) };
   my $error  = $@;
   my $exit   = $? ? ($? >> 8) : 0;
   my $signal = $? ? ($? & 127) : 0;
@@ -83,6 +104,8 @@ sub script_compiles {
     $error eq '' and $rv and $exit == 0 and $signal == 0 and $stderr =~ /syntax OK\s+\z/si
   );
 
+  File::Path::rmtree($dir);
+
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix compiles" );
   $test->diag( "$exit - $stderr" ) unless $ok;
@@ -92,6 +115,29 @@ sub script_compiles {
   return $ok;
 }
 
+# this is noticably slower for long @INC lists (sometimes present in cpantesters
+# boxes) than the previous implementation, which added a -I for every element in
+# @INC.  (also slower for more reasonable @INCs, but not noticably).  But it is
+# safer as very long argument lists can break calls to system
+sub _preload_module
+{
+  my $dir = File::Temp::tempdir( CLEANUP => 1 );
+  # this is hopefully a pm file that nobody would use
+  my $filename = File::Spec->catfile($dir, '__TEST_SCRIPT__.pm');
+  my $fh;
+  open($fh, ">$filename") 
+    || die "unable to open $filename: $!";
+  print($fh 'unshift @INC, ',
+    join ',', 
+    # quotemeta is overkill, but it will make sure that characters
+    # like " are quoted
+    map { '"' . quotemeta . '"' }
+    grep { ! ref } @INC)
+      || die "unable to write $filename: $!";
+  close($fh) || die "unable to close $filename: $!";;
+  $dir;
+}
+
 
 my $stdout;
 my $stderr;
@@ -101,16 +147,18 @@ sub script_runs {
   my $opt    = _options(\@_);
   my $unix   = shift @$args;
   my $path   = path( $unix );
-  my @libs   = map { "-I$_" } grep {!ref($_)} @INC;
-  my $cmd    = [ perl, @libs, $path, @$args ];
+  my $dir    = _preload_module();
+  my $cmd    = [ perl, "-I$dir", '-M__TEST_SCRIPT__', $path, @$args ];
      $stdout = '';
      $stderr = '';
-  my $rv     = eval { IPC::Run3::run3( $cmd, $opt->{stdin}, $opt->{stdout}, $opt->{stderr} ) };
+  my $rv     = eval { run3( $cmd, $opt->{stdin}, $opt->{stdout}, $opt->{stderr} ) };
   my $error  = $@;
   my $exit   = $? ? ($? >> 8) : 0;
   my $signal = $? ? ($? & 127) : 0;
   my $ok     = !! ( $error eq '' and $rv and $exit == $opt->{exit} and $signal == $opt->{signal} );
 
+  File::Path::rmtree($dir);
+
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix runs" );
   $test->diag( "$exit - $stderr" ) unless $ok;
@@ -267,15 +315,15 @@ Test::Script - Basic cross-platform tests for scripts
 
 =head1 VERSION
 
-version 1.09
+version 1.12
 
 =head1 SYNOPSIS
 
  use Test::More tests => 2;
  use Test::Script;
  
- script_compiles('script/awesomescript.pl');
- script_runs(['script/awesomescript.pl', '--awesome-argument']);
+ script_compiles('script/myscript.pl');
+ script_runs(['script/myscript.pl', '--my-argument']);
 
 =head1 DESCRIPTION
 
@@ -395,14 +443,14 @@ Same as C<stdout> above, except for stderr.
 
 =head2 script_stdout_is
 
- script_stdout_is $expected_stdout, $test;
+ script_stdout_is $expected_stdout, $test_name;
 
 Tests if the output to stdout from the previous L</script_runs> matches the 
 expected value exactly.
 
 =head2 script_stdout_isnt
 
- script_stdout_is $expected_stdout, $test;
+ script_stdout_is $expected_stdout, $test_name;
 
 Tests if the output to stdout from the previous L</script_runs> does NOT match the 
 expected value exactly.
@@ -423,14 +471,14 @@ expression.
 
 =head2 script_stderr_is
 
- script_stderr_is $expected_stderr, $test;
+ script_stderr_is $expected_stderr, $test_name;
 
 Tests if the output to stderr from the previous L</script_runs> matches the 
 expected value exactly.
 
 =head2 script_stderr_isnt
 
- script_stderr_is $expected_stderr, $test;
+ script_stderr_is $expected_stderr, $test_name;
 
 Tests if the output to stderr from the previous L</script_runs> does NOT match the 
 expected value exactly.
@@ -449,12 +497,25 @@ expression.
 Tests if the output to stderr from the previous L</script_runs> does NOT match the regular
 expression.
 
+=head1 CAVEATS
+
+This module is fully supported back to Perl 5.8.1.  It may work on 5.8.0.
+It should work on Perl 5.6.x and I may even test on 5.6.2.  I will accept
+patches to maintain compatibility for such older Perls, but you may
+need to fix it on 5.6.x / 5.8.0 and send me a patch.
+
+This module uses L<IPC::Run3> to compile and run scripts.  There are a number of
+outstanding issues with this module, and maintenance for L<IPC::Run3> is not swift.
+One of these is that L<IPC::Run3> incorrectly throws an exception on Windows when
+you feed it a Perl script with a compile error.  Currently L<Test::Script> probes
+for this bug (it checks for the bug, not for a specific version) and applies a
+workaround in that case.  I am hoping to remove the work around once the bug is
+fixed in L<IPC::Run3>.
+
 =head1 SEE ALSO
 
 L<Test::Script::Run>, L<Test::More>
 
-=cut
-
 =head1 AUTHOR
 
 Original author: Adam Kennedy
